@@ -1,34 +1,32 @@
 from __future__ import absolute_import
-from pony.py23compat import imap
+from pony.py23compat import PY2, imap, basestring, buffer, int_types
 
-from decimal import Decimal, InvalidOperation
+from decimal import Decimal
 from datetime import datetime, date, time, timedelta
 from uuid import UUID
 
 import warnings
 warnings.filterwarnings('ignore', '^Table.+already exists$', Warning, '^pony\\.orm\\.dbapiprovider$')
 
-try:
-    import MySQLdb as mysql_module
-except ImportError as e:
-    try:
-        import pymysql as mysql_module
-    except ImportError:
-        raise ImportError('No module named MySQLdb or pymysql found')
+try: import MySQLdb as mysql_module
+except ImportError:
+    try: import pymysql as mysql_module
+    except ImportError: raise ImportError('No module named MySQLdb or pymysql found')
     else:
         import pymysql.converters as mysql_converters
         from pymysql.constants import FIELD_TYPE, FLAG, CLIENT
-        mysql_converters.encoders[buffer] = lambda val: mysql_converters.escape_str(str(val))
+        if PY2:
+            mysql_converters.encoders[buffer] = lambda val: mysql_converters.escape_str(str(val))
         mysql_converters.encoders[timedelta] = lambda val: mysql_converters.escape_str(timedelta2str(val))
         mysql_module_name = 'pymysql'
 else:
     import MySQLdb.converters as mysql_converters
     from MySQLdb.constants import FIELD_TYPE, FLAG, CLIENT
     mysql_module_name = 'MySQLdb'
-    from _mysql import string_literal
+    from MySQLdb import string_literal
 
 from pony.orm import core, dbschema, dbapiprovider
-from pony.orm.core import log_orm, log_sql, OperationalError
+from pony.orm.core import log_orm, OperationalError
 from pony.orm.dbapiprovider import DBAPIProvider, Pool, get_version_tuple, wrap_dbapi_exceptions
 from pony.orm.sqltranslation import SQLTranslator
 from pony.orm.sqlbuilding import SQLBuilder, join
@@ -88,20 +86,11 @@ class MySQLBuilder(SQLBuilder):
             return 'DATE_SUB(', builder(expr), ", INTERVAL '", timedelta2str(delta), "' HOUR_SECOND)"
         return 'SUBTIME(', builder(expr), ', ', builder(delta), ')'
 
-def _string_sql_type(converter):
-    result = 'VARCHAR(%d)' % converter.max_len if converter.max_len else 'LONGTEXT'
-    if converter.db_encoding: result += ' CHARACTER SET %s' % converter.db_encoding
-    return result
-
-class MySQLUnicodeConverter(dbapiprovider.UnicodeConverter):
-    sql_type = _string_sql_type
-
 class MySQLStrConverter(dbapiprovider.StrConverter):
-    sql_type = _string_sql_type
-
-class MySQLLongConverter(dbapiprovider.IntConverter):
     def sql_type(converter):
-        return 'BIGINT'
+        result = 'VARCHAR(%d)' % converter.max_len if converter.max_len else 'LONGTEXT'
+        if converter.db_encoding: result += ' CHARACTER SET %s' % converter.db_encoding
+        return result
 
 class MySQLRealConverter(dbapiprovider.RealConverter):
     def sql_type(converter):
@@ -141,26 +130,27 @@ class MySQLProvider(DBAPIProvider):
     select_for_update_nowait_syntax = False
     max_time_precision = default_time_precision = 0
     varchar_default_max_len = 255
+    uint64_support = True
 
     dbapi_module = mysql_module
     dbschema_cls = MySQLSchema
     translator_cls = MySQLTranslator
     sqlbuilder_cls = MySQLBuilder
 
+    fk_types = { 'SERIAL' : 'BIGINT UNSIGNED' }
+
     converter_classes = [
         (bool, dbapiprovider.BoolConverter),
-        (unicode, MySQLUnicodeConverter),
-        (str, MySQLStrConverter),
-        (int, dbapiprovider.IntConverter),
-        (long, MySQLLongConverter),
+        (basestring, MySQLStrConverter),
+        (int_types, dbapiprovider.IntConverter),
         (float, MySQLRealConverter),
         (Decimal, dbapiprovider.DecimalConverter),
-        (buffer, MySQLBlobConverter),
         (datetime, dbapiprovider.DatetimeConverter),
         (date, dbapiprovider.DateConverter),
         (time, MySQLTimeConverter),
         (timedelta, MySQLTimedeltaConverter),
         (UUID, MySQLUuidConverter),
+        (buffer, MySQLBlobConverter),
     ]
 
     def normalize_name(provider, name):
