@@ -1,4 +1,5 @@
 from __future__ import absolute_import
+from pony.py23compat import PY2, basestring, unicode, buffer, int_types
 
 from decimal import Decimal
 from datetime import datetime, date, time, timedelta
@@ -10,12 +11,11 @@ from psycopg2 import extensions
 import psycopg2.extras
 psycopg2.extras.register_uuid()
 
-from pony.orm import core, dbschema, sqlbuilding, dbapiprovider
+from pony.orm import core, dbschema, dbapiprovider
 from pony.orm.core import log_orm
-from pony.orm.dbapiprovider import DBAPIProvider, Pool, ProgrammingError, wrap_dbapi_exceptions
+from pony.orm.dbapiprovider import DBAPIProvider, Pool, wrap_dbapi_exceptions
 from pony.orm.sqltranslation import SQLTranslator
-from pony.orm.sqlbuilding import Value
-from pony.utils import throw
+from pony.orm.sqlbuilding import Value, SQLBuilder
 from pony.converting import timedelta2str
 
 class PGColumn(dbschema.Column):
@@ -34,13 +34,14 @@ class PGValue(Value):
         value = self.value
         if isinstance(value, bool): return value and 'true' or 'false'
         return Value.__unicode__(self)
+    if not PY2: __str__ = __unicode__
 
-class PGSQLBuilder(sqlbuilding.SQLBuilder):
+class PGSQLBuilder(SQLBuilder):
     dialect = 'PostgreSQL'
     make_value = PGValue
     def INSERT(builder, table_name, columns, values, returning=None):
         if not values: result = [ 'INSERT INTO ', builder.quote_name(table_name) ,' DEFAULT VALUES' ]
-        else: result = sqlbuilding.SQLBuilder.INSERT(builder, table_name, columns, values)
+        else: result = SQLBuilder.INSERT(builder, table_name, columns, values)
         if returning is not None: result.extend([' RETURNING ', builder.quote_name(returning) ])
         return result
     def TO_INT(builder, expr):
@@ -66,25 +67,17 @@ class PGSQLBuilder(sqlbuilding.SQLBuilder):
             return '(', builder(expr), " - INTERVAL '", timedelta2str(delta), "' DAY TO SECOND)"
         return '(', builder(expr), ' - ', builder(delta), ')'
 
-class PGUnicodeConverter(dbapiprovider.UnicodeConverter):
-    def py2sql(converter, val):
-        return val.encode('utf-8')
-    def sql2py(converter, val):
-        if isinstance(val, unicode): return val
-        return val.decode('utf-8')
-
 class PGStrConverter(dbapiprovider.StrConverter):
-    def py2sql(converter, val):
-        return val.decode(converter.encoding).encode('utf-8')
-    def sql2py(converter, val):
-        if not isinstance(val, unicode):
-            if converter.utf8: return val
-            val = val.decode('utf-8')
-        return val.encode(converter.encoding, 'replace')
+    if PY2:
+        def py2sql(converter, val):
+            return val.encode('utf-8')
+        def sql2py(converter, val):
+            if isinstance(val, unicode): return val
+            return val.decode('utf-8')
 
-class PGLongConverter(dbapiprovider.IntConverter):
-    def sql_type(converter):
-        return 'BIGINT'
+class PGIntConverter(dbapiprovider.IntConverter):
+    signed_types = {None: 'INTEGER', 8: 'SMALLINT', 16: 'SMALLINT', 24: 'INTEGER', 32: 'INTEGER', 64: 'BIGINT'}
+    unsigned_types = {None: 'INTEGER', 8: 'SMALLINT', 16: 'INTEGER', 24: 'INTEGER', 32: 'BIGINT'}
 
 class PGRealConverter(dbapiprovider.RealConverter):
     def sql_type(converter):
@@ -138,6 +131,8 @@ class PGProvider(DBAPIProvider):
 
     default_schema_name = 'public'
 
+    fk_types = { 'SERIAL' : 'INTEGER', 'BIGSERIAL' : 'BIGINT' }
+
     def normalize_name(provider, name):
         return name[:provider.max_name_len].lower()
 
@@ -173,7 +168,7 @@ class PGProvider(DBAPIProvider):
 
     @wrap_dbapi_exceptions
     def execute(provider, cursor, sql, arguments=None, returning_id=False):
-        if isinstance(sql, unicode): sql = sql.encode('utf8')
+        if PY2 and isinstance(sql, unicode): sql = sql.encode('utf8')
         if type(arguments) is list:
             assert arguments and not returning_id
             cursor.executemany(sql, arguments)
@@ -235,18 +230,16 @@ class PGProvider(DBAPIProvider):
 
     converter_classes = [
         (bool, dbapiprovider.BoolConverter),
-        (unicode, PGUnicodeConverter),
-        (str, PGStrConverter),
-        (long, PGLongConverter),
-        (int, dbapiprovider.IntConverter),
+        (basestring, PGStrConverter),
+        (int_types, PGIntConverter),
         (float, PGRealConverter),
         (Decimal, dbapiprovider.DecimalConverter),
-        (buffer, PGBlobConverter),
         (datetime, PGDatetimeConverter),
         (date, dbapiprovider.DateConverter),
         (time, dbapiprovider.TimeConverter),
         (timedelta, PGTimedeltaConverter),
         (UUID, PGUuidConverter),
+        (buffer, PGBlobConverter),
     ]
 
 provider_cls = PGProvider
