@@ -1,5 +1,5 @@
 from __future__ import absolute_import, print_function, division
-from pony.py23compat import izip, imap, itervalues
+from pony.py23compat import PY2, izip, imap, itervalues, basestring, unicode, buffer
 
 from operator import attrgetter
 from decimal import Decimal
@@ -26,6 +26,7 @@ class Param(object):
         elif paramstyle == 'named': return u':p%d' % param.id
         elif paramstyle == 'pyformat': return u'%%(p%d)s' % param.id
         else: throw(NotImplementedError)
+    if not PY2: __str__ = __unicode__
     def __repr__(param):
         return '%s(%r)' % (param.__class__.__name__, param.paramkey)
 
@@ -38,12 +39,17 @@ class Value(object):
         value = self.value
         if value is None: return 'null'
         if isinstance(value, bool): return value and '1' or '0'
-        if isinstance(value, (int, long, float, Decimal)): return str(value)
         if isinstance(value, basestring): return self.quote_str(value)
         if isinstance(value, datetime): return self.quote_str(datetime2timestamp(value))
         if isinstance(value, date): return self.quote_str(str(value))
-        if isinstance(value, buffer): return "X'%s'" % hexlify(value)
-        assert False, value
+        if PY2:
+            if isinstance(value, (int, long, float, Decimal)): return str(value)
+            if isinstance(value, buffer): return "X'%s'" % hexlify(value)
+        else:
+            if isinstance(value, (int, float, Decimal)): return str(value)
+            if isinstance(value, bytes): return "X'%s'" % hexlify(value).decode('ascii')
+        assert False, value  # pragma: no cover
+    if not PY2: __str__ = __unicode__
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self.value)
     def quote_str(self, s):
@@ -382,7 +388,7 @@ class SQLBuilder(object):
         expr_list = [ builder(expr) for expr in x ]
         return builder(expr1), ' IN (', join(', ', expr_list), ')'
     def NOT_IN(builder, expr1, x):
-        if not x: throw(AstError, 'Empty IN clause')
+        if not x: return '1 = 1'
         if len(x) >= 1 and x[0] == 'SELECT':
             return builder(expr1), ' NOT IN ', builder(x)
         expr_list = [ builder(expr) for expr in x ]
@@ -410,15 +416,15 @@ class SQLBuilder(object):
     LENGTH = make_unary_func('length')
     ABS = make_unary_func('abs')
     def COALESCE(builder, *args):
-        if len(args) < 2: assert False
+        if len(args) < 2: assert False  # pragma: no cover
         return 'coalesce(', join(', ', imap(builder, args)), ')'
     def MIN(builder, *args):
-        if len(args) == 0: assert False
+        if len(args) == 0: assert False  # pragma: no cover
         elif len(args) == 1: fname = 'MIN'
         else: fname = 'least'
         return fname, '(',  join(', ', imap(builder, args)), ')'
     def MAX(builder, *args):
-        if len(args) == 0: assert False
+        if len(args) == 0: assert False  # pragma: no cover
         elif len(args) == 1: fname = 'MAX'
         else: fname = 'greatest'
         return fname, '(',  join(', ', imap(builder, args)), ')'
